package john111898.ld30;

import java.awt.image.BufferedImage;
import java.lang.reflect.Constructor;

import javax.swing.JFrame;

import john111898.ld30.control.ControlsManager;
import john111898.ld30.control.KeyboardHandler;
import john111898.ld30.control.MouseHandler;
import john111898.ld30.gui.GUI;
import john111898.ld30.gui.GUIGameOver;
import john111898.ld30.gui.GUIGoals;
import john111898.ld30.gui.GUIItemSelect;
import john111898.ld30.gui.GUIItemSelectMoreInfo;
import john111898.ld30.gui.GUIManager;
import john111898.ld30.gui.GUIResources;
import john111898.ld30.gui.GUITileAlert;
import john111898.ld30.gui.GUITop;
import john111898.ld30.gui.elements.IGUIObject;
import john111898.ld30.render.RenderComponent;
import john111898.ld30.resources.ResourceCount;
import john111898.ld30.resources.ResourceRequirements;
import john111898.ld30.tile.Tile;
import john111898.ld30.tile.TileAsphaltRoad;
import john111898.ld30.tile.TileDirtRoad;
import john111898.ld30.tile.TileGrass;
import john111898.ld30.tile.TileSelectList;
import john111898.ld30.tile.TileStoneRoad;
import john111898.ld30.tile.buildings.TileAdvancedToolsmith;
import john111898.ld30.tile.buildings.TileAluminumMine;
import john111898.ld30.tile.buildings.TileCarbonFiberManufactory;
import john111898.ld30.tile.buildings.TileCeramicsFactory;
import john111898.ld30.tile.buildings.TileCityCenterIndustrial;
import john111898.ld30.tile.buildings.TileCityCenterIron;
import john111898.ld30.tile.buildings.TileCityCenterSpace;
import john111898.ld30.tile.buildings.TileCityCenterStone;
import john111898.ld30.tile.buildings.TileCoalMine;
import john111898.ld30.tile.buildings.TileConcreteProducer;
import john111898.ld30.tile.buildings.TileElectronicsFactory;
import john111898.ld30.tile.buildings.TileForest;
import john111898.ld30.tile.buildings.TileFuelRefinery;
import john111898.ld30.tile.buildings.TileGlassGatherer;
import john111898.ld30.tile.buildings.TileGoldMine;
import john111898.ld30.tile.buildings.TileIndustrialLogger;
import john111898.ld30.tile.buildings.TileIronMine;
import john111898.ld30.tile.buildings.TileLargeHouse;
import john111898.ld30.tile.buildings.TileMediumHouse;
import john111898.ld30.tile.buildings.TileOilWell;
import john111898.ld30.tile.buildings.TilePlasticsFactory;
import john111898.ld30.tile.buildings.TileRecycler;
import john111898.ld30.tile.buildings.TileSignalDisrupter;
import john111898.ld30.tile.buildings.TileSignalDisrupterII;
import john111898.ld30.tile.buildings.TileSignalEnhancer;
import john111898.ld30.tile.buildings.TileSignalEnhancerII;
import john111898.ld30.tile.buildings.TileSmallHouse;
import john111898.ld30.tile.buildings.TileSpacecraftFactory;
import john111898.ld30.tile.buildings.TileSteelFactory;
import john111898.ld30.tile.buildings.TileStoneMine;
import john111898.ld30.tile.buildings.TileTitaniumMine;
import john111898.ld30.tile.buildings.TileToolsmith;
import john111898.ld30.tile.buildings.TileVeryLargeHouse;
import john111898.ld30.util.ImageLoader;
import john111898.ld30.util.SoundManager;
import john111898.ld30.world.SimulatedEnemy;
import john111898.ld30.world.World;

public class Game {
	private GameThread g;
	private RenderThread r;
	
	private JFrame window;
	private RenderComponent render;
	
	
	
	private SoundManager sounds;
	private World world;
	
	private GUIManager guis;
	
	private GUI tileAlert;
	private GUITop top;
	private GUIResources resourceGUI;
	
	private Tile tileWaiting;
	private int tileWaitingX;
	private int tileWaitingY;
	
	private Tile currentTileType = new TileDirtRoad(0,0);
	
	ResourceCount player = new ResourceCount(ResourceCount.playerStart);
	ResourceCount enemy = new ResourceCount(ResourceCount.playerStart);
	ResourceCount universal = new ResourceCount(ResourceCount.resourcePool);
	
	SimulatedEnemy simEnemy;
	
	int timeCounter = 1;
	
	public static final int AGE_STONE = 0;
	public static final int AGE_IRON = 1;
	public static final int AGE_INDUSTRIAL = 2;
	public static final int AGE_SPACE = 3;
	int currentAge;
	
	public Game() {
		init();
	}
	
	public void init() {		
		Main.tileSpritesheet = ImageLoader.loadImage("tileSpritesheet");
		Main.guiBack192x192 = ImageLoader.loadImage("guiBack192x192");
		Main.buttonYes44x44 = ImageLoader.loadImage("buttonYes44x44");
		Main.buttonNo44x44 = ImageLoader.loadImage("buttonNo44x44");
		Main.guiTop = ImageLoader.loadImage("guiTop");
		Main.topSpacer = ImageLoader.loadImage("topSpacer");
		Main.buttonExit = ImageLoader.loadImage("buttonExit");
		Main.buildingButtonsSpritesheet = ImageLoader.loadImage("buildingButtonsSpritesheet");
		Main.buttonResources = ImageLoader.loadImage("buttonResources");
		Main.buttonGoals = ImageLoader.loadImage("buttonGoals");
		Main.guiBackItemSelect = ImageLoader.loadImage("guiBackItemSelect");
		Main.guiBackItemSelectMoreInfo = ImageLoader.loadImage("guiBackItemSelectMoreInfo");
		Main.buttonCloseMenu = ImageLoader.loadImage("buttonCloseMenu");
		Main.buttonMoreInfo = ImageLoader.loadImage("buttonMoreInfo");
		Main.guiBackResources = ImageLoader.loadImage("guiBackResources");
		Main.buttonResEnemy = ImageLoader.loadImage("buttonResEnemy");
		Main.buttonResYou = ImageLoader.loadImage("buttonResYou");
		Main.buttonResUniversal = ImageLoader.loadImage("buttonResUniversal");
		
		world = new World();
		sounds = new SoundManager();
		guis = new GUIManager();
		ControlsManager.instance = this;
		
		render = new RenderComponent(world, guis);
		
		window = new JFrame();
		window.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		window.setTitle(Main.NAME);
		window.setSize(Main.SCREEN_WIDTH, Main.SCREEN_HEIGHT);
		window.add(render);
		window.addKeyListener(new KeyboardHandler());
		window.addMouseListener(new MouseHandler());
		window.addMouseMotionListener(new MouseHandler());
		window.addMouseWheelListener(new MouseHandler());
		window.setResizable(false);
		window.setVisible(true);
		window.setIconImage(getIconFromSpritesheet());
		
		top = new GUITop();
		guis.addTopGUI(top);
		
		resourceGUI = new GUIResources();
		
		g = new GameThread(this);
		r = new RenderThread(this);
		g.start();
		r.start();
		
		currentAge = AGE_STONE;
		top.updateAge("Stone");
		simEnemy = new SimulatedEnemy(AGE_STONE, SimulatedEnemy.NORMAL);
	}
	
	public BufferedImage getIconFromSpritesheet() {
		//return Main.tileSpritesheet.getSubimage(0, 0, 32, 32);
		return Main.buildingButtonsSpritesheet.getSubimage(0,192,48,48); 
	}
	
	public void updateGame() {
		if (ControlsManager.keysPressed[0]) {
			if (Main.viewOffsetY < Main.viewOffsetYMax) {
				Main.viewOffsetY += 10;
			}
		}
		if (ControlsManager.keysPressed[1]) {
			if (Main.viewOffsetY > Main.viewOffsetYMin) {
				Main.viewOffsetY -= 10;
			}
		}
		if (ControlsManager.keysPressed[2]) {
			if (Main.viewOffsetX > Main.viewOffsetXMin) {
				Main.viewOffsetX -= 10;
			}
		}
		if (ControlsManager.keysPressed[3]) {
			if (Main.viewOffsetX < Main.viewOffsetXMax) {
				Main.viewOffsetX += 10;
			}
		}
		
		world.updateTilesLogic(Main.mouseX, Main.mouseY);
		if (timeCounter == 20 * 30) {
			world.addTileResourceProduction(player, universal);
			if (guis.activeGUI.id.equals("resources")) {
				resourceGUI.setResources(player);
			} else {
				if (guis.activeGUI.id.equals("goals")) {
					guis.removeGUI();
					switch(currentAge) {
					case 0:
						guis.addGUI(new GUIGoals(player, new ResourceCount(ResourceCount.goalAdvanceIron)));
						break;
					case 1:
						guis.addGUI(new GUIGoals(player, new ResourceCount(ResourceCount.goalAdvanceIndustrial)));
						break;
					case 2:
						guis.addGUI(new GUIGoals(player, new ResourceCount(ResourceCount.goalAdvanceSpace)));
						break;
					default:
						guis.addGUI(new GUIGoals(player, new ResourceCount(ResourceCount.empty)));
						break;
					}
				}
			}
			simEnemy.addResourceProduction(enemy, universal, world.getDisrupters(), world.getDisruptersII());
			if (simEnemy.spaceshipReady()) {
				endGame(false);
			}
			timeCounter = 0;
		}
		timeCounter++;
		testForNewAge();
	}
	
	public void updateRender() {
		render.repaint();
	}
	
	public void leftClicked(int x, int y) {
		IGUIObject o = guis.click(x, y);
		if (o != null) {
			if (o.getGUI().id.equals("tilealert")) {
				if (o.getObjectInfo().equals("button-yes")) {
					ResourceRequirements r = tileWaiting.resources;
					boolean canBuild = true;
					int[] playerNums = new int[] {player.numWood, player.numStone, player.numGlass, player.numIron, player.numGold, player.numCoal, player.numAluminum, player.numOil, player.numFuel, player.numSteel, player.numCeramics, player.numPlastics, player.numTitanium, player.numElectronics, player.numCarbonFiber, player.numConcrete};
					String[] resourceNames = new String[] {"Wood", "Stone", "Glass", "Iron", "Gold", "Coal", "Aluminum", "Oil", "Fuel", "Steel", "Ceramics", "Plastics", "Titanium", "Electronics", "Carbon Fiber", "Concrete"};
					int r1Num = 0;
					int r2Num = 0;
					int r3Num = 0;
					if (r.resource1) {
						for (int i = 0; i < resourceNames.length; i++) {
							if (resourceNames[i].equals(r.r1.name)) {
								r1Num = i;
							}
						}
						if (r.resource2) {
							for (int i = 0; i < resourceNames.length; i++) {
								if (resourceNames[i].equals(r.r2.name)) {
									r2Num = i;
								}
							}
							if (r.resource3) {
								for (int i = 0; i < resourceNames.length; i++) {
									if (resourceNames[i].equals(r.r3.name)) {
										r3Num = i;
									}
								}
								if (!(playerNums[r1Num] >= r.numR1 && playerNums[r2Num] >= r.numR2 && playerNums[r3Num] >= r.numR3)) {
									canBuild = false;
								} else {
									player.removeByID(r1Num, r.numR1);
									player.removeByID(r2Num, r.numR2);
									player.removeByID(r3Num, r.numR3);
								}
							} else {
								if (!(playerNums[r1Num] >= r.numR1 && playerNums[r2Num] >= r.numR2)) {
									canBuild = false;
								} else {
									player.removeByID(r1Num, r.numR1);
									player.removeByID(r2Num, r.numR2);
								}
							}
						} else {
							if (!(playerNums[r1Num] >= r.numR1)) {
								canBuild = false;
							} else {
								player.removeByID(r1Num, r.numR1);
							}
						}
					}
					if (canBuild) {
						world.setTile(tileWaiting, tileWaitingX, tileWaitingY);
						sounds.playSound("buttonPress.wav", 0.0f);
					} else {
						sounds.playSound("buttonPressNo.wav", 0.0f);
					}
				} else {
					sounds.playSound("buttonPressNo.wav", 0.0f);
				}
				tileWaiting = null;
				tileWaitingX = 0;
				tileWaitingY = 0;
				guis.removeGUI();
			} else {
				if (o.getGUI().id.equals("top")) {
					switch(o.getObjectInfo()) {
					case "button-exit":
						sounds.playSound("buttonPressNo.wav", 0.0f);
						System.exit(0);
						break;
					case "button-road":
						if (guis.activeGUI.id.equals("null")) {
							switch(currentAge) {
							case 0:
								guis.addGUI(new GUIItemSelect(new Tile[] {new TileGrass(0,0), new TileDirtRoad(0,0)}, "Roads"));
								break;
							case 1:
								guis.addGUI(new GUIItemSelect(new Tile[] {new TileGrass(0,0), new TileDirtRoad(0,0), new TileStoneRoad(0,0)}, "Roads"));
								break;
							default:
								guis.addGUI(new GUIItemSelect(new Tile[] {new TileGrass(0,0), new TileDirtRoad(0,0), new TileStoneRoad(0,0), new TileAsphaltRoad(0,0)}, "Roads"));
								break;
							}
						}
						break;
					case "button-stone":
						guis.addGUI(new GUIItemSelect(new Tile[] {new TileCityCenterStone(0,0), new TileForest(0,0), new TileStoneMine(0,0), new TileToolsmith(0,0), new TileGlassGatherer(0,0), new TileSmallHouse(0,0)}, "Stone Age Tiles"));
						break;
					case "button-iron":
						if (currentAge >= 1) {
							guis.addGUI(new GUIItemSelect(new Tile[] {new TileCityCenterIron(0,0), new TileIronMine(0,0), new TileGoldMine(0,0), new TileCoalMine(0,0), new TileAdvancedToolsmith(0,0), new TileMediumHouse(0,0)}, "Iron Age Tiles"));
						}
						break;
					case "button-industrial":
						if (currentAge >= 2) {
							guis.addGUI(new GUIItemSelect(new Tile[] {new TileCityCenterIndustrial(0,0), new TileAluminumMine(0,0), new TileOilWell(0,0), new TileFuelRefinery(0,0), new TileIndustrialLogger(0,0), new TileSignalDisrupter(0,0), new TileSignalEnhancer(0,0), new TileSteelFactory(0,0), new TileCeramicsFactory(0,0), new TilePlasticsFactory(0,0), new TileLargeHouse(0,0)}, "Industrial Age Tiles"));
						}
						break;
					case "button-space":
						if (currentAge >= 3) {
							guis.addGUI(new GUIItemSelect(new Tile[] {new TileCityCenterSpace(0,0), new TileTitaniumMine(0,0), new TileSignalDisrupterII(0,0), new TileSignalEnhancerII(0,0), new TileElectronicsFactory(0,0), new TileRecycler(0,0), new TileCarbonFiberManufactory(0,0), new TileConcreteProducer(0,0), new TileSpacecraftFactory(0,0), new TileVeryLargeHouse(0,0)}, "Space Age Tiles"));
						}
						break;
					case "button-resources":
						if (guis.activeGUI.id.equals("null")) {
							resourceGUI.setResources(player);
							guis.addGUI(resourceGUI);
						}
						break;
					case "button-goals":
						if (guis.activeGUI.id.equals("null")) {
							switch(currentAge) {
							case 0:
								guis.addGUI(new GUIGoals(player, new ResourceCount(ResourceCount.goalAdvanceIron)));
								break;
							case 1:
								guis.addGUI(new GUIGoals(player, new ResourceCount(ResourceCount.goalAdvanceIndustrial)));
								break;
							case 2:
								guis.addGUI(new GUIGoals(player, new ResourceCount(ResourceCount.goalAdvanceSpace)));
								break;
							default:
								guis.addGUI(new GUIGoals(player, new ResourceCount(ResourceCount.empty)));
								break;
							}
						}
						break;
					}
				} else {
					if (o.getGUI().id.equals("select")) {
						if (o.getObjectInfo() != "button-close" && o.getObjectInfo() != "button-moreinfo") {
							String tileName = o.getObjectInfo().substring(7);
							Tile t = TileSelectList.getTileFromString(tileName);
							top.updateCurrentTile(t);
							currentTileType = t;
						} else {
							if (o.getObjectInfo() == "button-moreinfo") {
								guis.addSecondary(new GUIItemSelectMoreInfo(710, 75, "moreInfo", currentTileType));
							} else {
								guis.removeGUI();
								guis.removeSecondary();
								sounds.playSound("buttonPressNo.wav", 0.0f);
							}
						}
					} else {
						if (o.getGUI().id.equals("moreInfo")) {
							guis.removeSecondary();
							sounds.playSound("buttonPressNo.wav", 0.0f);
						} else {
							if (o.getGUI().id.equals("resources")) {
								switch(o.getObjectInfo()) {
								case "button-close":
									guis.removeGUI();
									sounds.playSound("buttonPressNo.wav", 0.0f);
									break;
								case "button-you":
									resourceGUI.setResources(player);
									break;
								case "button-enemy":
									resourceGUI.setResources(enemy);
									break;
								case "button-universal":
									resourceGUI.setResources(universal);
									break;
								}
							} else {
								if (o.getGUI().id.equals("goals")) {
									guis.removeGUI();
									sounds.playSound("buttonPressNo.wav", 0.0f);
								}
							}
						}
					}
				}
			}
		}
		Tile t = world.getClickedTile(x, y);
		if (t != null) {
			if (guis.activeGUI.id.equals("null")) {
				setupGUITileAlert();
				Class<?> c = currentTileType.getClass();		
				try {
					Constructor<?>[] construct = c.getConstructors();
					tileWaiting = (Tile) construct[0].newInstance(t.tileX, t.tileY);
					tileWaitingX = t.tileX;
					tileWaitingY = t.tileY;
				} catch (Exception e) {
					e.printStackTrace();
				} 
			}
		}
	}
	
	public void rightClicked(int x, int y) {
		IGUIObject o = guis.click(x, y);
		if (o != null) {
			
			return;
		}
		Tile t = world.getClickedTile(x, y);
		if (t != null) {
			
		}
	}
	
	public void setupGUITileAlert() {
		tileAlert = new GUITileAlert();
		guis.addGUI(tileAlert);
	}
	
	public void newAge() {
		sounds.playSound("newAge.wav", 0.0f);
		switch(currentAge) {
		case AGE_STONE:
			currentAge = AGE_IRON;
			top.updateAge("Iron");
			world.progressToIron();
			simEnemy.setAge(AGE_IRON);
			return;
		case AGE_IRON:
			currentAge = AGE_INDUSTRIAL;
			top.updateAge("Industrial");
			world.progressToIndustrial();
			simEnemy.setAge(AGE_INDUSTRIAL);
			return;
		case AGE_INDUSTRIAL:
			currentAge = AGE_SPACE;
			top.updateAge("Space");
			world.progressToSpace();
			simEnemy.setAge(AGE_SPACE);
			return;
		case AGE_SPACE:
			currentAge++;
			endGame(true);
			return;
		}
	}
	
	public void endGame(boolean playerSpaceFirst) {
		guis.removeGUI();
		guis.removeSecondary();
		guis.addGUI(new GUIGameOver(playerSpaceFirst, world, player, simEnemy.getBuildingsValue(), enemy));
	}
	
	public void testForNewAge() {
		switch(currentAge) {
		case 0:
			int[] goalNums = ResourceCount.goalAdvanceIron;
			int[] playerNums = new int[] {player.numWood, player.numStone, player.numGlass, player.numIron, player.numGold, player.numCoal, player.numAluminum, player.numOil, player.numFuel, player.numSteel, player.numCeramics, player.numPlastics, player.numTitanium, player.numElectronics, player.numCarbonFiber, player.numConcrete};
			boolean advance = true;
			for (int i = 0; i < 16; i++) {
				if (goalNums[i] > 0) {
					if (!(playerNums[i] >= goalNums[i])) {
						advance = false;
					}
				}
			}
			if (advance) {
				newAge();
			}
			break;
		case 1:
			int[] goalNums2 = ResourceCount.goalAdvanceIndustrial;
			int[] playerNums2 = new int[] {player.numWood, player.numStone, player.numGlass, player.numIron, player.numGold, player.numCoal, player.numAluminum, player.numOil, player.numFuel, player.numSteel, player.numCeramics, player.numPlastics, player.numTitanium, player.numElectronics, player.numCarbonFiber, player.numConcrete};
			boolean advance2 = true;
			for (int i = 0; i < 16; i++) {
				if (goalNums2[i] > 0) {
					if (!(playerNums2[i] >= goalNums2[i])) {
						advance2 = false;
					}
				}
			}
			if (advance2) {
				newAge();
			}
			break;
		case 2:
			int[] goalNums3 = ResourceCount.goalAdvanceSpace;
			int[] playerNums3 = new int[] {player.numWood, player.numStone, player.numGlass, player.numIron, player.numGold, player.numCoal, player.numAluminum, player.numOil, player.numFuel, player.numSteel, player.numCeramics, player.numPlastics, player.numTitanium, player.numElectronics, player.numCarbonFiber, player.numConcrete};
			boolean advance3 = true;
			for (int i = 0; i < 16; i++) {
				if (goalNums3[i] > 0) {
					if (!(playerNums3[i] >= goalNums3[i])) {
						advance3 = false;
					}
				}
			}
			if (advance3) {
				newAge();
			}
			break;
		case 3:
			if (world.checkForSpacecraft()) {
				newAge();
			}
			break;
		}
	}
}