package john111898.ld30.world;

import java.awt.Graphics;

import john111898.ld30.resources.ResourceCount;
import john111898.ld30.tile.Tile;
import john111898.ld30.tile.TileAsphaltRoad;
import john111898.ld30.tile.TileDirtRoad;
import john111898.ld30.tile.TileGrass;
import john111898.ld30.tile.TileStoneRoad;
import john111898.ld30.tile.buildings.TileCityCenterIndustrial;
import john111898.ld30.tile.buildings.TileCityCenterIron;
import john111898.ld30.tile.buildings.TileCityCenterSpace;
import john111898.ld30.tile.buildings.TileCityCenterStone;
import john111898.ld30.tile.buildings.TileSignalDisrupter;
import john111898.ld30.tile.buildings.TileSignalDisrupterII;
import john111898.ld30.tile.buildings.TileSpacecraftFactory;

public class World {
	Tile[][] tiles = new Tile[64][64];
	
	public World() {
		init();
	}
	
	private void init() {
		for (int i = 0; i < tiles.length; i++) {
			for (int j = 0; j < tiles[0].length; j++) {
				tiles[i][j] = new TileGrass(i, j);
			}
		}
	}
	
	public void setTile(Tile t, int tileX, int tileY) {
		tiles[tileX][tileY] = t;
	}
	
	public Tile getTile(int tileX, int tileY) {
		return tiles[tileX][tileY];
	}
	
	public void drawAllTiles(Graphics g) {
		for (int i = 0; i < tiles.length; i++) {
			for (int j = 0; j < tiles[0].length; j++) {
				Tile t = tiles[i][j];
				if (t.isVisible()) {
					t.draw(g);
				}
			}
		}
	}
	
	public void updateTilesLogic(int mouseX, int mouseY) {
		for (int i = 0; i < tiles.length; i++) {
			for (int j = 0; j < tiles[0].length; j++) {
				Tile t = tiles[i][j];
				t.isMouseOver(mouseX, mouseY);
			}
		}
	}
	
	public Tile getClickedTile(int mouseX, int mouseY) {
		for (int i = 0; i < tiles.length; i++) {
			for (int j = 0; j < tiles[0].length; j++) {
				Tile t = tiles[i][j];
				if (t.isMouseOver(mouseX, mouseY) && t.isVisible()) {
					return t;
				}
			}
		}
		return null;
	}
	
	public void progressToIron() {
		for (int i = 0; i < tiles.length; i++) {
			for (int j = 0; j < tiles[0].length; j++) {
				Tile t = tiles[i][j];
				if (t instanceof TileCityCenterStone) {
					t = new TileCityCenterIron(i, j);
				} else {
					if (t instanceof TileDirtRoad) {
						t = new TileStoneRoad(i, j);
					}
				}
			}
		}
	}
	
	public void progressToIndustrial() {
		for (int i = 0; i < tiles.length; i++) {
			for (int j = 0; j < tiles[0].length; j++) {
				Tile t = tiles[i][j];
				if (t instanceof TileCityCenterIron) {
					t = new TileCityCenterIndustrial(i, j);
				} else {
					if (t instanceof TileStoneRoad) {
						t = new TileAsphaltRoad(i, j);
					}
				}
			}
		}
	}
	
	public void progressToSpace() {
		for (int i = 0; i < tiles.length; i++) {
			for (int j = 0; j < tiles[0].length; j++) {
				Tile t = tiles[i][j];
				if (t instanceof TileCityCenterIndustrial) {
					t = new TileCityCenterSpace(i, j);
				}
			}
		}
	}
	
	public void addTileResourceProduction(ResourceCount player, ResourceCount universal) {
		int[] resourceModifications = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0};
		for (int i = 0; i < tiles.length; i++) {
			for (int j = 0; j < tiles.length; j++) {
				Tile t = tiles[i][j];
				if (t.output.resource != null) {
					switch(t.output.resource.name) {
					case "Wood":
						resourceModifications[0] += t.output.countPerMinute;
						break;
					case "Stone":
						resourceModifications[1] += t.output.countPerMinute;
						break;
					case "Glass":
						resourceModifications[2] += t.output.countPerMinute;
						break;
					case "Iron":
						resourceModifications[3] += t.output.countPerMinute;
						break;
					case "Gold":
						resourceModifications[4] += t.output.countPerMinute;
						break;
					case "Coal":
						resourceModifications[5] += t.output.countPerMinute;
						break;
					case "Aluminum":
						resourceModifications[6] += t.output.countPerMinute;
						break;
					case "Oil":
						resourceModifications[7] += t.output.countPerMinute;
						break;
					case "Fuel":
						resourceModifications[8] += t.output.countPerMinute;
						break;
					case "Steel":
						resourceModifications[9] += t.output.countPerMinute;
						break;
					case "Ceramics":
						resourceModifications[10] += t.output.countPerMinute;
						break;
					case "Plastics":
						resourceModifications[11] += t.output.countPerMinute;
						break;
					case "Titanium":
						resourceModifications[12] += t.output.countPerMinute;
						break;
					case "Electronics":
						resourceModifications[13] += t.output.countPerMinute;
						break;
					case "Carbon Fiber":
						resourceModifications[14] += t.output.countPerMinute;
						break;
					case "Concrete":
						resourceModifications[15] += t.output.countPerMinute;
						break;
					}
				}
			}
		}
		int boost = 0;
		for (int i = 0; i < tiles.length; i++) {
			for (int j = 0; j < tiles.length; j++) {
				Tile t = tiles[i][j];
				if (t.output.percentBoost != 0) {
					boost += t.output.percentBoost;
				}
			}
		}
		for (int i = 0; i < resourceModifications.length; i++) {
			resourceModifications[i] = resourceModifications[i] + (int) ((float) (boost / 100) * resourceModifications[i]);
		}
		if (universal.numWood >= resourceModifications[0]) {
			player.numWood += resourceModifications[0];
			universal.numWood -= resourceModifications[0];
		}
		if (universal.numStone >= resourceModifications[1]) {
			player.numStone += resourceModifications[1];
			universal.numStone -= resourceModifications[1];
		}
		if (universal.numGlass >= resourceModifications[2]) {
			player.numGlass += resourceModifications[2];
			universal.numGlass -= resourceModifications[2];
		}
		if (universal.numIron >= resourceModifications[3]) {
			player.numIron += resourceModifications[3];
			universal.numIron -= resourceModifications[3];
		}
		if (universal.numGold >= resourceModifications[4]) {
			player.numGold += resourceModifications[4];
			universal.numGold -= resourceModifications[4];
		}
		if (universal.numCoal >= resourceModifications[5]) {
			player.numCoal += resourceModifications[5];
			universal.numCoal -= resourceModifications[5];
		}
		if (universal.numAluminum >= resourceModifications[6]) {
			player.numAluminum += resourceModifications[6];
			universal.numAluminum -= resourceModifications[6];
		}
		if (universal.numOil >= resourceModifications[7]) {
			player.numOil += resourceModifications[7];
			universal.numOil -= resourceModifications[7];
		}
		if (universal.numFuel >= resourceModifications[8]) {
			player.numFuel += resourceModifications[8];
			universal.numFuel -= resourceModifications[8];
		}
		if (universal.numSteel >= resourceModifications[9]) {
			player.numSteel += resourceModifications[9];
			universal.numSteel -= resourceModifications[9];
		}
		if (universal.numCeramics >= resourceModifications[10]) {
			player.numCeramics += resourceModifications[10];
			universal.numCeramics -= resourceModifications[10];
		}
		if (universal.numPlastics >= resourceModifications[11]) {
			player.numPlastics += resourceModifications[11];
			universal.numPlastics -= resourceModifications[11];
		}
		if (universal.numTitanium >= resourceModifications[12]) {
			player.numTitanium += resourceModifications[12];
			universal.numTitanium -= resourceModifications[12];
		}
		if (universal.numElectronics >= resourceModifications[13]) {
			player.numElectronics += resourceModifications[13];
			universal.numElectronics -= resourceModifications[13];
		}
		if (universal.numCarbonFiber >= resourceModifications[14]) {
			player.numCarbonFiber += resourceModifications[14];
			universal.numCarbonFiber -= resourceModifications[14];
		}
		if (universal.numConcrete >= resourceModifications[15]) {
			player.numConcrete += resourceModifications[15];
			universal.numConcrete -= resourceModifications[15];
		}
	}
	
	public boolean checkForSpacecraft() {
		for (Tile[] t : tiles) {
			for (Tile tile : t) {
				if (tile instanceof TileSpacecraftFactory) {
					return true;
				}
			}
		}
		return false;
	}
	
	public Tile[][] getTiles() {
		return tiles;
	}
	
	public int getDisrupters() {
		int count = 0;
		for (Tile[] t : tiles) {
			for (Tile tile : t) {
				if (tile instanceof TileSignalDisrupter) {
					count++;
				}
			}
		}
		return count;
	}
	
	public int getDisruptersII() {
		int count = 0;
		for (Tile[] t : tiles) {
			for (Tile tile : t) {
				if (tile instanceof TileSignalDisrupterII) {
					count++;
				}
			}
		}
		return count;
	}
}